==> locator_lodestones-1.4.1%2B1.21.6%2BA-sources.jar: net/pneumono/locator_lodestones/WaypointNameRendering.java <==
package net.pneumono.locator_lodestones;

import net.minecraft.class_11200;
import net.minecraft.class_2561;
import net.minecraft.class_310;
import net.minecraft.class_327;
import net.minecraft.class_332;
import net.minecraft.class_3532;
import net.minecraft.class_8012;
import net.minecraft.class_9779;
import net.minecraft.class_9848;
import net.pneumono.locator_lodestones.config.ConfigManager;

import java.util.Optional;

//? if >=1.21.9 {
/*import net.minecraft.entity.Entity;
import net.minecraft.world.World;
import net.minecraft.world.waypoint.EntityTickProgress;
*///?}

public class WaypointNameRendering {
    public static void renderNames(class_310 client, class_332 context, class_9779 tickCounter, int centerY) {
        if (!ConfigManager.tabShowsNames() || !client.field_1690.field_1907.method_1434()) return;

        //? if >=1.21.9 {
        /*Entity cameraEntity = client.getCameraEntity();
        if (cameraEntity == null) return;
        World world = cameraEntity.getEntityWorld();
        EntityTickProgress entityTickProgress = (tickedEntity) -> tickCounter.getTickProgress(
                !world.getTickManager().shouldSkipTick(tickedEntity)
        );
        *///?}

        class_11200 bestWaypoint = null;
        double bestYaw = 61;
        for (class_11200 waypoint : WaypointTracking.CURRENT_WAYPOINTS) {
            //? if >=1.21.9 {
            /*double yaw = waypoint.getRelativeYaw(client.world, client.gameRenderer.getCamera(), entityTickProgress);
            *///?} else {
            double yaw = waypoint.method_71153(client.field_1687, client.field_1773.method_19418());
            //?}
            double absYaw = Math.abs(yaw);
            if (absYaw < Math.abs(bestYaw)) {
                bestYaw = yaw;
                bestWaypoint = waypoint;
            }
        }

        if (bestWaypoint != null) {
            Optional<class_2561> textOptional = WaypointTracking.WAYPOINT_NAMES.get(bestWaypoint.method_70763());
            if (textOptional.isPresent()) {
                class_2561 text = textOptional.get();
                class_327 textRenderer = client.field_1772;

                int width = textRenderer.method_27525(text);
                int x = getXFromYaw(context, bestYaw) - width / 2;

                context.method_25294(x + 5 - 2, centerY - 10 - 2, x + width + 5 + 2, centerY - 10 + 9 + 2, class_9848.method_71346(0.5F, class_8012.field_42974));

                context.method_27535(
                        textRenderer,
                        text,
                        x + 5,
                        centerY - 10,
                        class_8012.field_42973
                );
            }
        }
    }

    private static int getXFromYaw(class_332 context, double relativeYaw) {
        return class_3532.method_15386((context.method_51421() - 9) / 2.0F) + (int)(relativeYaw * 173.0 / 2.0 / 60.0);
    }
}

==> locator_lodestones-1.4.2%2B1.21.9%2BA-sources.jar: net/pneumono/locator_lodestones/WaypointNameRendering.java <==
package net.pneumono.locator_lodestones;

import net.minecraft.class_11200;
import net.minecraft.class_11728;
import net.minecraft.class_1297;
import net.minecraft.class_1937;
import net.minecraft.class_2561;
import net.minecraft.class_310;
import net.minecraft.class_327;
import net.minecraft.class_332;
import net.minecraft.class_3532;
import net.minecraft.class_8012;
import net.minecraft.class_9779;
import net.minecraft.class_9848;
import net.pneumono.locator_lodestones.config.ConfigManager;

import java.util.Optional;

public class WaypointNameRendering {
    public static void renderNames(class_310 client, class_332 context, class_9779 tickCounter, int centerY) {
        if (!ConfigManager.tabShowsNames() || !client.field_1690.field_1907.method_1434()) return;

        //? if >=1.21.9 {
        class_1297 cameraEntity = client.method_1560();
        if (cameraEntity == null) return;
        class_1937 world = cameraEntity.method_73183();
        class_11728 entityTickProgress = (tickedEntity) -> tickCounter.method_60637(
                !world.method_54719().method_54746(tickedEntity)
        );
        //?}

        class_11200 bestWaypoint = null;
        double bestYaw = 61;
        for (class_11200 waypoint : WaypointTracking.getWaypoints()) {
            //? if >=1.21.9 {
            double yaw = waypoint.method_71153(client.field_1687, client.field_1773.method_19418(), entityTickProgress);
            //?} else {
            /*double yaw = waypoint.getRelativeYaw(client.world, client.gameRenderer.getCamera());
            *///?}
            double absYaw = Math.abs(yaw);
            if (absYaw < Math.abs(bestYaw)) {
                bestYaw = yaw;
                bestWaypoint = waypoint;
            }
        }

        if (bestWaypoint != null) {
            Optional<class_2561> textOptional = WaypointTracking.getWaypointName(bestWaypoint.method_70763());
            if (textOptional.isPresent()) {
                class_2561 text = textOptional.get();
                class_327 textRenderer = client.field_1772;

                int width = textRenderer.method_27525(text);
                int x = getXFromYaw(context, bestYaw) - width / 2;

                context.method_25294(x + 5 - 2, centerY - 10 - 2, x + width + 5 + 2, centerY - 10 + 9 + 2, class_9848.method_71346(0.5F, class_8012.field_42974));

                context.method_27535(
                        textRenderer,
                        text,
                        x + 5,
                        centerY - 10,
                        class_8012.field_42973
                );
            }
        }
    }

    private static int getXFromYaw(class_332 context, double relativeYaw) {
        return class_3532.method_15386((context.method_51421() - 9) / 2.0F) + (int)(relativeYaw * 173.0 / 2.0 / 60.0);
    }
}
